﻿/**
* 円
* @author nutsu
* @version 0.2
*/

package{
	
	import flash.display.Graphics;
	import flash.geom.Point;
	
	public class Circle {
		
		//半径
		public var r:Number;
		
		//中心点
		public var x:Number;
		public var y:Number;
		
		public function Circle( r_:Number, x_:Number=0.0, y_:Number=0.0 ){
			r = r_;
			x = x_;
			y = y_;
		}
		
		/**
		* 角度tでの位置
		* @param	ラジアン
		* @return	Point
		*/
		public function getPosByRad( t:Number ):Point{
			return new Point( x + r*Math.cos(t), y + r*Math.sin(t) );
		}
		
		/**
		* 直線との交点
		* @param	直線
		* @return	Point[]
		*/
		public function intersectionByLine( sl:Line ):Array{
			//直線と中心の距離ベクトル
			var dv:Point = sl.vectorFromPoint( new Point(x,y) );
			
			if( dv.length>r ){
				//距離が半径より大きい場合、交点なし
				return [];
			}else if( dv.length<r ){
				//交点が２つ
				var drad:Number;
				var irad:Number;
				if( dv.length>0 ){
					//距離ベクトルの角度
					drad = Math.atan2( dv.y, dv.x );
					//距離ベクトルから交点の角度
					irad = Math.acos( dv.length/r );
				}else{
					drad = Math.atan2( sl.b, sl.a );
					irad = Math.PI/2;
				}
				return [ getPosByRad( drad+irad ) , getPosByRad(drad-irad) ];
			}else{
				//接線
				dv.offset( x, y );
				return [ dv ];
			}
		}
		
		/**
		* 円との交点
		* @param	Circle
		* @return	Point[]
		*/
		public function intersectionByCircle( c:Circle ):Array{
			//中心間ベクトル
			var dv:Point = new Point( c.x-x, c.y-y );
			//距離
			var len:Number = dv.length;
			
			if( len==0 || len>( r+c.r ) || len<Math.abs(r-c.r) ){
				//交点なし
				return [];
			}else if( len<( r+c.r ) ){
				//交点は２つ
				//ベクトル上のｘ
				var t:Number = 0.5*( len*len + r*r - c.r*c.r )/len;
				//ベクトルからのラジアン
				var a1:Number = Math.acos( t/r );
				//中心ベクトルのラジアン
				var a0:Number = Math.atan2( dv.y, dv.x );
				return [ getPosByRad( a0+a1 ) , getPosByRad( a0-a1 ) ];
			}else{
				//接している
				dv.normalize( r );
				dv.offset( x, y );
				return [dv];
			}
		}
		
		
		/**
		* 描画
		* @param	Graphics
		* @return
		*/
		public function draw( g:Graphics ):void{
			g.drawCircle( x, y, r );
		}
		
		public function toString():String{
			return "[Circle r="+r+" x="+x+" y="+y+" ]";
		}
	}
	
}
